#ifndef _CHRONOPTICS_TOF_CSF_WRITER_H_
#define _CHRONOPTICS_TOF_CSF_WRITER_H_

#include <chronoptics/tof/camera.h>
#include <chronoptics/tof/camera_config.h>
#include <chronoptics/tof/calibration.h>
#include <chronoptics/tof/data.h>

#ifdef __cplusplus
extern "C" {
#endif

/** The csf writer class takes care of writing depth frames to disk.  The class
 * has to be created from either a Camera or csf reader, because it needs
 * information about the camera that was used to record the depth frames. 
 * Therefore you should only write depth frames coming from one camera into the
 * same writer

 */
typedef struct tof_csf_writer* tof_csf_writer_t;

typedef struct tof_csf_writer const* tof_csf_writer_ct;

/** Destruct tof_csf_writer */
TOF_EXPORT void tof_csf_writer_delete(tof_csf_writer_t ptr);

/** Construct the csf writer from a camera
 * @param ptr Pointer to class
 * @param file_location Path to where the csf file should be created
 * @param camera The camera to get the camera config and calibration file from
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_writer_t tof_csf_writer_new_from_camera(const char* file_location, tof_camera_ct camera, tof_error_t *error);

/** Construct csf writer from calibration and configuration
 * @param ptr Pointer to class
 * @param file_location Path to where the csf file should be created
 * @param camera_config Camera configuration
 * @param calibration Calibration
 * @param serial Serial number of camera
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_csf_writer_t tof_csf_writer_new_from_raw_data(const char* file_location, tof_camera_config_ct camera_config, tof_calibration_ct calibration, const char* serial, tof_error_t *error);

/** Write frame to csf file
 * @param ptr Pointer to class
 * @param frame A data frame
 * @param error Pointer to error
 */
TOF_EXPORT void tof_csf_writer_write_frame(tof_csf_writer_t ptr, tof_data_ct frame, tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
