#ifndef _CHRONOPTICS_TOF_TUI_CAMERA_HPP_
#define _CHRONOPTICS_TOF_TUI_CAMERA_HPP_

#include <chronoptics/tof/tui_camera.h>

#include <chronoptics/tof/camera.hpp>
#include <chronoptics/tof/camera_config.hpp>

namespace chronoptics {
namespace tof {

/** The main interface to the tui camera
*/
class TuiCamera : public LiveCamera {
 public:
  /** Construct from pointer */
  TuiCamera(tof_tui_camera_t ptr = nullptr) : LiveCamera(reinterpret_cast<tof_live_camera_t>(ptr)) {}

  /** Construct the tui camera
   * @param serial The serial number of the camera to connect to, if the camera
   * is available over both usb and gige it will be connected over usb. If an
   * empty string is supplied the camera will connect to the first usb camera,
   * if no usb camera is connected it will connect to the first gige camera that
   * replies (this reply time is random, so you might connect to a different
   * camera each time if you've got multiple cameras connected to your network).
   */
  TuiCamera(StringView serial) : TuiCamera(tof_tui_camera_new(serial, TOF_ERROR_HANDLER{})) {}

  /** Set GPIO mode for a certain pin
   * @param pin Pin
   * @param gpio_mode GPIO Mode
   */
  void set_gpio_mode(size_t pin, GpioMode gpio_mode) {
    auto this_class = reinterpret_cast<tof_tui_camera_t>(this->ptr_);
    return tof_tui_camera_set_gpio_mode(this_class, pin, static_cast<tof_gpio_mode>(gpio_mode), TOF_ERROR_HANDLER{});
  }

  /** Set trigger mode
   * @param trigger_mode Trigger mode
   */
  void set_trigger_mode(TriggerMode trigger_mode) {
    auto this_class = reinterpret_cast<tof_tui_camera_t>(this->ptr_);
    return tof_tui_camera_set_trigger_mode(this_class, static_cast<tof_trigger_mode>(trigger_mode), TOF_ERROR_HANDLER{});
  }

};

/** Construct a tui camera using the gige interface. This could be useful in two
 * cases. The first is when the camera is connected with both usb and ethernet
 * but you want to use ethernet. The second is when for some reason the default
 * maximum packet size is too large.
 * @param serial The serial number of the camera to connect to. If an empty
 * string is supplied the camera will connect to the first gige camera that
 * replies (this reply time is random, so you might connect to a different
 * camera each time if you've got multiple cameras connected to your network).
 * @param packet_size The packet size to use for communication over ethernet
 * @return A tui camera
 */
inline TuiCamera create_tui_camera_gige(StringView serial, uint16_t packet_size = 1472) {
  TuiCamera new_tui_camera(static_cast<tof_tui_camera_t>(nullptr));
  auto ptr = reinterpret_cast<tof_tui_camera_t*>(&new_tui_camera);
  *ptr = tof_create_tui_camera_gige(serial, packet_size, TOF_ERROR_HANDLER{});
  return new_tui_camera;
}

} // tof
} // chronoptics

#endif
